#include <genesis.h>

/**
 *  PSGの停止/再生を制御するための定数
 */
typedef enum {
    MUSIC_STOP, // 停止
    MUSIC_PLAY, // 再生
} MusicState;

int main(u16 hard) {
    // 曲(トーン値と発音時間の組)
    u16 musicalScore[8][2] = {
        {273, 10},
        {0, 50},
        {273, 10},
        {0, 50},
        {273, 10},
        {0, 50},
        {52, 10}
    };
    // 曲の再生位置
    u16 index = 0;
    // 曲の状態(停止/再生)
    u16 state = MUSIC_PLAY;
    // PSGを再生するチャンネル番号
    u8 channel = 0;
    // 次の曲再生までの待ち時間
    u16 interval = 120;
    // 時間判定用の減算カウンタ
    u8 counter = 0;

    // エンベロープレベルの設定
    PSG_setEnvelope(channel, PSG_ENVELOPE_MAX);

    VDP_drawText("SOUND ONLY", 15, 13);

    // メインループ
    while (TRUE) {
        SYS_doVBlankProcess();

        // 曲が停止状態
        if (state != MUSIC_PLAY) {
            // 曲の状態を初期化
            state = MUSIC_PLAY;
            index = 0;
            // カウンタに再生待ち時間を設定
            counter = interval;
        }

        // 再生/停止の残り時間を減らす
        if (counter) {
            counter--;
        }
        // 残り時間カウンタが0
        else {
            // 曲の最後まで来ていれば曲を停止する
            if (index >= sizeof(musicalScore) / sizeof(musicalScore[0])) {
                state = MUSIC_STOP;
                PSG_setTone(channel, 0);
                continue;
            }

            // 指定のトーン値を鳴らし、発音時間をカウンタに設定する
            u16 tone = musicalScore[index][0];
            counter = musicalScore[index][1];
            PSG_setTone(channel, tone);

            // 曲の再生位置を進める
            index++;
        }
    }

    return 0;
}